Index: apps/app_fax.c
===================================================================
--- apps/app_fax.c	(revision 186715)
+++ apps/app_fax.c	(working copy)
@@ -14,6 +14,7 @@
 
 /*** MODULEINFO
 	 <depend>spandsp</depend>
+	 <defaultenabled>yes</defaultenabled>
 ***/
  
 #include "asterisk.h"
@@ -28,10 +29,8 @@
 #include <errno.h>
 #include <tiffio.h>
 
+#define SPANDSP_EXPOSE_INTERNAL_STRUCTURES
 #include <spandsp.h>
-#ifdef HAVE_SPANDSP_EXPOSE_H
-#include <spandsp/expose.h>
-#endif
 #include <spandsp/version.h>
 
 #include "asterisk/lock.h"
@@ -433,11 +432,22 @@
 	ast_activate_generator(s->chan, &generator, &fax);
 
 	while (!s->finished) {
-		res = ast_waitfor(s->chan, 20);
-		if (res < 0)
+
+		/* Watchdog */
+		now = ast_tvnow();
+		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
+			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
+			res = -1;
 			break;
-		else if (res > 0)
+		}
+
+		res = ast_waitfor(s->chan, 50);
+		if (res > 0)
 			res = 0;
+		else if (!res)
+			continue;
+		else if (res < 0)
+			break;
 
 		inf = ast_read(s->chan);
 		if (inf == NULL) {
@@ -496,14 +506,6 @@
 
 		ast_frfree(inf);
 		inf = NULL;
-
-		/* Watchdog */
-		now = ast_tvnow();
-		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
-			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
-			res = -1;
-			break;
-		}
 	}
 
 	ast_debug(1, "Loop finished, res=%d\n", res);
@@ -586,16 +588,26 @@
 
 	while (!s->finished) {
 
+		/* Watchdog */
+		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
+			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
+			res = -1;
+			break;
+		}
+
 		res = ast_waitfor(s->chan, 20);
 		if (res < 0)
 			break;
-		else if (res > 0)
-			res = 0;
 
 		last_frame = now;
 		now = ast_tvnow();
 		t38_terminal_send_timeout(&t38, ast_tvdiff_us(now, last_frame) / (1000000 / 8000));
 
+		if (res > 0)
+			res = 0;
+		else if (!res)
+			continue;
+
 		inf = ast_read(s->chan);
 		if (inf == NULL) {
 			ast_debug(1, "Channel hangup\n");
@@ -627,13 +639,6 @@
 
 		ast_frfree(inf);
 		inf = NULL;
-
-		/* Watchdog */
-		if (ast_tvdiff_sec(now, start) > WATCHDOG_TOTAL_TIMEOUT || ast_tvdiff_sec(now, state_change) > WATCHDOG_STATE_TIMEOUT) {
-			ast_log(LOG_WARNING, "It looks like we hung. Aborting.\n");
-			res = -1;
-			break;
-		}
 	}
 
 	ast_debug(1, "Loop finished, res=%d\n", res);
@@ -658,7 +663,7 @@
 
 	pbx_builtin_setvar_helper(s->chan, "FAXMODE", NULL);
 	pbx_builtin_setvar_helper(s->chan, "REMOTESTATIONID", NULL);
-	pbx_builtin_setvar_helper(s->chan, "FAXPAGES", NULL);
+	pbx_builtin_setvar_helper(s->chan, "FAXPAGES", "0");
 	pbx_builtin_setvar_helper(s->chan, "FAXRESOLUTION", NULL);
 	pbx_builtin_setvar_helper(s->chan, "FAXBITRATE", NULL); 
 
